/*********************************************
This program was produced by the
CodeWizardAVR V1.24.0 Standard
Automatic Program Generator
 Copyright 1998-2003 HP InfoTech s.r.l.
http://www.hpinfotech.ro
e-mail:office@hpinfotech.ro

Project : PPE Robot solaire
Version : 1
Date    : 17/08/2009
Author  : MNO                             
Company : Lyce Pierre Emile Martin 18000 BOURGES                                
Comments: Mise en oeuvre du protocole I2C sur l'interface TWI
Transfert de l'tat des BP sur les LEDs de la carte (PCF8574)
Adaptation de la note d'application AVR315 d'ATMEL
Lecture de la ddp batterie sur la carte MD23
Commande des moteurs de la carte MD23

Chip type           : ATmega8535
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 128
*********************************************/

#include <mega8535.h>
#include <string.h>
#include <stdlib.h>
#include <MD23.h>
#include <delay.h>  

// Redfinition du buffer TWI dfini dans le fichier MD23.h
#define TWI_BUFFER_SIZE 	20

#include "TWI_F.c" // Fonctions du drivers TWI adaptes pour CVAVR 
/***********************************************************************
Project : PPE Robot(s)
Version : 1
Date    : 12/08/2009
Author  : MNO
File : twi_f.c
Description : Fonctions du drivers TWI (I2C), dveloppes par ATMEL
et dcrites dans la note d'application AVR315, adaptes au
cross compilateur CVAVR.
Fichier  inclure en en tte d'une application ncessitant une interface
2 fil I2C.
Evolution:  remplacer par une bibliothque
***********************************************************************/


#include <TWI_Master_CVAVR.h>  


// Declare your global variables here 
//-----------------------------------------------------------------------------------------------------------------------------
static unsigned char TWI_msgSize;             		// Number of bytes to be transmitted.       
static unsigned char TWI_buf[TWI_BUFFER_SIZE];    	// Transceiver buffer
static unsigned char TWI_state = TWI_NO_STATE;      	// State byte. Default set to TWI_NO_STATE 
 
union TWI_STR                      			// Status byte holding flags.
{
    unsigned char all;
    unsigned char lastTransOK:1;      
    unsigned char unusedBits:7; 
} TWI_statusReg ={0};  
//-----------------------------------------------------------------------------------------------------------------------------

//-----------------------------------------------------------------------------------------------------------------------------
// Fonctions du drivers TWI
//-----------------------------------------------------------------------------------------------------------------------------
void TWI_Master_Initialise(void)
{                                               // Adapte pour CVAVR
  TWBR = TWI_TWBR;                              // Set bit rate register (Baudrate). Defined in header file.
// TWSR = TWI_TWPS;                             // Not used. Driver presumes prescaler to be 00.
  TWDR = 0xFF;                                  // Default content = SDA released.
  						//  TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
  TWCR = 0b00000100;                            // (1<<TWEN)| Enable TWI-interface and release TWI pins.
                               			// (0<<TWIE)|(0<<TWINT)|Disable Interupt.
                               			// (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| No Signal requests.
                                         	// 0<<TWWC); 
} 
//-----------------------------------------------------------------------------------------------------------------------------
unsigned char TWI_Transceiver_Busy(void)
{                                               // Adapte pour CVAVR
  return ( TWCR & 0b00000001);                  // ( TWCR & (1<<TWIE) ) IF TWI Interrupt is enabled then the Transceiver is busy
}
//-----------------------------------------------------------------------------------------------------------------------------
void TWI_Start_Transc_With_Data( unsigned char *msg, unsigned char msgSize )
{
  unsigned char temp;
                                                // Adapte pour CVAVR
  while ( TWI_Transceiver_Busy() );             // Wait until TWI is ready for next transmission.

  TWI_msgSize = msgSize;                        // Number of data to transmit.
  TWI_buf[0]  = msg[0];                         // Store slave address with R/W setting.
  if (!( msg[0] & 0b00000001 ))       		// (!( msg[0] & (TRUE<<TWI_READ_BIT) ))
  {						// If it is a write operation, then also copy data.
    for ( temp = 1; temp < msgSize; temp++ )
      TWI_buf[ temp ] = msg[ temp ];
  }
  TWI_statusReg.all = 0;                        // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  TWI_state         = TWI_NO_STATE ;            //  TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
  TWCR =  0b10100101;                           // (1<<TWEN)| TWI Interface enabled.
                           			// (1<<TWIE)|(1<<TWINT)|Enable TWI Interupt and clear the flag.
                				// (0<<TWEA)|(1<<TWSTA)|(0<<TWSTO)|Initiate a START condition!!!!!.
                                      		// (0<<TWWC);
} 
//-----------------------------------------------------------------------------------------------------------------------------
unsigned char TWI_Get_Data_From_Transceiver( unsigned char *msg, unsigned char msgSize )
{
  unsigned char i;
          
  while ( TWI_Transceiver_Busy() );             // Wait until TWI is ready for next transmission.

  if( TWI_statusReg.lastTransOK )               // Last transmission competed successfully.              
  {                                             
    for ( i=0; i<msgSize; i++ )                 // Copy data from Transceiver buffer.
    {
      msg[ i ] = TWI_buf[ i ];
    }
  }
  return( TWI_statusReg.lastTransOK );                                   
}  
//-----------------------------------------------------------------------------------------------------------------------------

// ----------------------------------------------------------------------------------------------------------------------------
// Interruption TWI
// ----------------------------------------------------------------------------------------------------------------------------
interrupt [TWI] void twi_isr(void)      // Adapte pour CVAVR
{					// #pragma vector=TWI_vect
					// __interrupt void TWI_ISR(void)
  static unsigned char TWI_bufPtr;
  
  switch (TWSR)
  {
    case TWI_START:             // START has been transmitted  
    
    case TWI_REP_START:         // Repeated START has been transmitted
      TWI_bufPtr = 0;   	// Set buffer pointer to the TWI Address location
      				
    case TWI_MTX_ADR_ACK:       // SLA+W has been tramsmitted and ACK received 
    
    case TWI_MTX_DATA_ACK:      // Data byte has been tramsmitted and ACK received
      if (TWI_bufPtr < TWI_msgSize)
      {
        TWDR = TWI_buf[TWI_bufPtr++];                     // TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
        TWCR = 0b10000101;                                // (1<<TWEN)| TWI Interface enabled
                                     			  // (1<<TWIE)|(1<<TWINT)|Enable TWI Interupt and clear the flag to send byte
                         				  // (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| 
               						  // (0<<TWWC); 
      }
      else                    // Send STOP after last byte
      {
        TWI_statusReg.lastTransOK = TRUE;                 // Set status bits to completed successfully.
         						  // TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE 
        TWCR = 0b10010100; 		                  // (1<<TWEN)| TWI Interface enabled
                                    			  // (0<<TWIE)|(1<<TWINT)| Disable TWI Interrupt and clear the flag
                         				  // (0<<TWEA)|(0<<TWSTA)|(1<<TWSTO)| Initiate a STOP condition.
                               				  // (0<<TWWC);
      }
    break;
      
    case TWI_MRX_DATA_ACK:      // Data byte has been received and ACK tramsmitted
      TWI_buf[TWI_bufPtr++] = TWDR; 
      
    case TWI_MRX_ADR_ACK:       // SLA+R has been tramsmitted and ACK received
      if (TWI_bufPtr < (TWI_msgSize-1) )                  // Detect the last byte to NACK it.
      {                                                   // TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
        TWCR = 0b10000101;                                // (1<<TWEN)| TWI Interface enabled
                                     			  // (1<<TWIE)|(1<<TWINT)| Enable TWI Interupt and clear the flag to read next byte
                          				  // (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|Send ACK after reception
                                                          // (0<<TWWC);  
      }else                    // Send NACK after next reception
      {						          // TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
        TWCR = 0b10000101;                                // (1<<TWEN)| TWI Interface enabled
                                     			  // (1<<TWIE)|(1<<TWINT)| Enable TWI Interupt and clear the flag to read next byte
                          				  // (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| Send NACK after reception
                                			  // (0<<TWWC); 
      }    
    break; 
      
    case TWI_MRX_DATA_NACK:     // Data byte has been received and NACK tramsmitted
      TWI_buf[TWI_bufPtr] = TWDR;
      TWI_statusReg.lastTransOK = TRUE;                 // Set status bits to completed successfully. 
      							// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
      TWCR = 0b10010100;                                // (1<<TWEN)| TWI Interface enabled
                                  			// (0<<TWIE)|(1<<TWINT)| Disable TWI Interrupt and clear the flag
                       					// (0<<TWEA)|(0<<TWSTA)|(1<<TWSTO)| Initiate a STOP condition.
                                             		// (0<<TWWC); 
    break;  
          
    case TWI_ARB_LOST:          // Arbitration lost                             
    							// TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
      TWCR = 0b10100101;                                // (1<<TWEN)| TWI Interface enabled
                                   			// (1<<TWIE)|(1<<TWINT)| Enable TWI Interupt and clear the flag
                        				// (0<<TWEA)|(1<<TWSTA)|(0<<TWSTO)| Initiate a (RE)START condition.
                                              		// (0<<TWWC);
    break;
      
    case TWI_MTX_ADR_NACK:      // SLA+W has been tramsmitted and NACK received
    case TWI_MRX_ADR_NACK:      // SLA+R has been tramsmitted and NACK received    
    case TWI_MTX_DATA_NACK:     // Data byte has been tramsmitted and NACK received
//    case TWI_NO_STATE              // No relevant state information available; TWINT = 0
    case TWI_BUS_ERROR:         // Bus error due to an illegal START or STOP condition  
    
    default:     
      TWI_state = TWSR;                                 // Store TWSR and automatically sets clears noErrors bit.
                                                        // Reset TWI Interface
                                                        // TWCR = TWINT TWEA TWSTA TWSTO TWWC TWEN - TWIE
      TWCR = 0b00000100;                                // (1<<TWEN)| Enable TWI-interface and release TWI pins
                                   			// (0<<TWIE)|(0<<TWINT)| Disable Interupt
                       					// (0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)| No Signal requests
                                              		// (0<<TWWC);
  }
} 


// Constantes en flash
// ==========================================================================================================
// ==================================== Messages pour le LCD I2C BATRON =====================================
// ==========================================================================================================
// Trames d'initialisation de l'afficheur LCD I2C BATRON (voir doc PCF2119x p22  p25)
/* ----------------------------------------------------------------------------------------------------------
Ordre   Dsignation					Valeur		Remarques
// ----------------------------------------------------------------------------------------------------------
1 	Adresse de l'afficheur 			=	0x76 		(SA0 = 1) ( envoyer en premier)
2 	Select function setting			=	0x00 		Doit prcder une ou plusieurs commandes 
									(dans la trame I2C !) doc PCF2119x p22
									voir doc PCF2119x p22
3 	Command : Function set			=	0x34            2 lignes de 16 caractres 
4 	Command : Display control		=	0x0c            Display on, curseur off
5 	Command : Entry mode set		=       0x06  		Incrment, no shift
6	Command : Extended instruction set	= 	0x35            ----------------------------------
7	Command : Set display configuration	= 	0x04  		left-to-right, top to bottom 
8	Command : Set coefficient temperature	= 	0x10            TC1 = 0, TC2 = 0 (Default)
9	Command : Set Internal HVgen stages	=	0x42            HV stage 3  ???
10	Command : Set VLCD			= 	0x9F            set VLCD, store VA (Rglage contraste)
11	Command : Normal instruction set	= 	0x34            ----------------------------------
12	Command : Set DDRRAM address		= 	0x80            DDRAM address = 0x00
----------------------------------------------------------------------------------------------------------*/
flash unsigned char LCD_BATRON_Init[13]={0x76,0x00,0x34,0x0c,0x06,0x35,0x04,0x10,0x42,0x9F,0x34,0x80,0x02};



// Fonctions
// ---------------------------------------------------------------------------------------------------------- 
     
  
 


// ----------------------------------------------------------------------------------------------------------
// Boucle principale
// ----------------------------------------------------------------------------------------------------------
void main(void)
{
// Declare your local variables here 
unsigned char Message_For_TWI_Buffer[20];
unsigned char Switch_All_LED_On[]={0x40,0x00}; 	// Pour clairer toutes les LED {addLed + W (0x40), data = all Led on}
unsigned char Switch_LED_On[]={0x40,0x00}; 	// Pour clairer des LED {addLed + W (0x40), data = Led on}
unsigned char Switch_All_LED_Off[]={0x40,0xff}; 	// Pour teindre toutes les LED {addLed + W (ox40), data = all Led off}
unsigned char Accueil_Lcd[]={0x76,0x40,'P','h','i','l','i','p','s'};
unsigned char Read_BP[]={0x43,0x00}; 		// Pour lire les BP {addBP +R (0x43), data=nothing} 
unsigned char Valeur_Registre_MD23; 
char Valeur_Registre_MD23_Texte[]={'','',''};

unsigned char Trame_LCD_ligne1[18] = {0x76,0x40,'R'+0x80,'v',0xA0,0xA0,'M','o','d',0xA0,0xA0,'U'+0x80,'='+0x80,
0xA0,0xA0,'.'+0x80,0xA0,0xd6};

// ==========================================================================================================
// ==================================== Messages pour le LCD I2C BATRON =====================================
// ==========================================================================================================
// 	Command : Clear display			= 	0x01   
// ----------------------------------------------------------------------------------------------------------
unsigned char LCD_BATRON_Clear[]={0x76,0x00,0x01}; 

// ----------------------------------------------------------------------------------------------------------



// ==========================================================================================================
// ==================================== Messages pour la carte MD23 =========================================
// ========================================================================================================== 
// Gnral
unsigned char Reset_MD23[]={MD23_ADRESSE_BASE,Reg_Commande, Reset_Encodeur};
unsigned char Mode0_MD23[]={MD23_ADRESSE_BASE,Reg_Mode,Mode2};
unsigned char Registre_MD23[]={0xff, 0};

// Vitesse des moteurs
unsigned char Speed1[]={MD23_ADRESSE_BASE,Reg_Speed1, 137};
unsigned char Speed2[]={MD23_ADRESSE_BASE,Reg_Speed2_Turn, 128};  
unsigned char Time_OUT_Desable[]={MD23_ADRESSE_BASE,Reg_Commande, Desactive_Timeout_Moteurs};

// Tension batterie   
unsigned char Read_UBatt_MD23_1[]={MD23_ADRESSE_BASE,Reg_VBatterie};//   add_SoftRev add_Enc1a    add_
unsigned char Read_UBatt_MD23_2[]={MD23_ADRESSE_BASE | 1, 0};
 


unsigned char i, valeur=128,taille_message;
// ==========================================================================================================

TWI_Master_Initialise(); 	// Initialisation de l'interface TWI en mode Standby

// Global enable interrupts
#asm("sei")

// ------------------------------------ Initialisations ---------------------------------------------------------
// --------------------------------------------------------------------------------------------------------------
TWI_Start_Transc_With_Data(Switch_All_LED_Off,sizeof(Switch_All_LED_Off)); 


// Copie de la table LCD_BATRON_Init dans le tableau de variables Message_For_TWI_Buffer
for (i=0;i<13;i++)
{
	Message_For_TWI_Buffer[i]=LCD_BATRON_Init[i];
} 
	       		
TWI_Start_Transc_With_Data(Message_For_TWI_Buffer,sizeof(LCD_BATRON_Init));                 


// Initialisation de la carte MD23
// --------------------------------------------------------------------------------------------------------------
TWI_Start_Transc_With_Data(Reset_MD23,sizeof(Reset_MD23));
TWI_Start_Transc_With_Data(Time_OUT_Desable,3);
TWI_Start_Transc_With_Data(Mode0_MD23,3);
TWI_Start_Transc_With_Data(Speed1,3);
TWI_Start_Transc_With_Data(Speed2,3);



// Test afficheur BATRON 
TWI_Start_Transc_With_Data(LCD_BATRON_Clear,sizeof(LCD_BATRON_Clear));
TWI_Start_Transc_With_Data(Accueil_Lcd,sizeof(Accueil_Lcd));    // Message d'accueil  
delay_ms(3000);
TWI_Start_Transc_With_Data(LCD_BATRON_Clear,sizeof(LCD_BATRON_Clear));
delay_ms(3000);

while (1)
      { 
// ----------------------------------  Lecture des donnes -------------------------------------------------------- 
// Lecture des BP
	TWI_Start_Transc_With_Data(Read_BP,1);  		// Envoi octet d'adresse des BP + bit RW=lecture(0x43)
        TWI_Get_Data_From_Transceiver(Read_BP, 2 );  		// Lecture du buffer (octet1=add, octet2=donnee etat BP) 

// Lecture des info de la carte MD23
// Tension d'alimentation  
      
        TWI_Start_Transc_With_Data(Read_UBatt_MD23_1,2);        //  Envoi octet d'adresse MD23 + dcalage + bit RW=criture
        TWI_Start_Transc_With_Data(Read_UBatt_MD23_2,1);        //  Envoi octet d'adresse MD23 +  bit RW=lecture
        TWI_Get_Data_From_Transceiver(Registre_MD23,2);  	//  Lecture tension alim de la carte MD23
        Valeur_Registre_MD23 = Registre_MD23[1];  
        
// ----------------------------------  Traitement ------------------------------------------------------------------        
        
        Switch_LED_On[1]=~Read_BP[1];                  		// Transfert etat BP dans message LED On
         
        itoa((unsigned int)Valeur_Registre_MD23,Valeur_Registre_MD23_Texte);     
        
        if(Valeur_Registre_MD23<10)
        {
         Trame_LCD_ligne1[2]=Valeur_Registre_MD23_Texte[0]+0x80;
         Trame_LCD_ligne1[3]=0xA0;
         Trame_LCD_ligne1[4]=0xA0;
        } 
        else if(Valeur_Registre_MD23<100)
        {
         Trame_LCD_ligne1[2]=Valeur_Registre_MD23_Texte[0]+0x80;
         Trame_LCD_ligne1[3]=Valeur_Registre_MD23_Texte[1]+0x80;
         Trame_LCD_ligne1[4]=0xA0;
        } 
        else 
         for (i=0;i<sizeof(Valeur_Registre_MD23_Texte);i++)
        {
         Trame_LCD_ligne1[13]=Valeur_Registre_MD23_Texte[0]+0x80;
         Trame_LCD_ligne1[14]=Valeur_Registre_MD23_Texte[1]+0x80;
         Trame_LCD_ligne1[16]=Valeur_Registre_MD23_Texte[2]+0x80;
        }

                
// ----------------------------------  Ecriture sur les LED et l'afficheur  -----------------------------------------
        TWI_Start_Transc_With_Data(LCD_BATRON_Clear,sizeof(LCD_BATRON_Clear));    
        TWI_Start_Transc_With_Data(Trame_LCD_ligne1,sizeof(Trame_LCD_ligne1));
        TWI_Start_Transc_With_Data(Switch_LED_On,2);   		// Commande des LED
        delay_ms(200);     
      };
}


